function [rf_filt] = modified_rainflow(RF,timepv,Tjpv,Tjfull,Ts)
%% Input
% RF: rainflow matrix
% timepv: time vector after peaks and valleys function
% Tjpv: temperature vector after peaks and valleys function
% Tjfull: Original temperature vector (before peaks and valleys function)
% Ts: sample time of the mission profile
% p.s.:Do not Truncate Temperature Data!
%% Output
% rf_filt(:,1): Cycles heating time
% rf_filt(:,2): Cycles amplitude
% rf_filt(:,3): Cycles mean value
% rf_filt(:,4): Number of cycles (0.5 or 1.0). 
% p.s.: only the last half cycle with increasing in temperatures are included
%%
timefull = (0:length(Tjfull)-1)*Ts;
%% Encontrando ultimos meio ciclos e salvando suas posies
descres = length(RF);
ni=0.5;
aux1 = 1;
while ni == 0.5
    ni = RF(descres,1);
    if ni == 0.5
        vetorNi(aux1) = descres;
    else
        vetorNi(aux1) = 0;
    end
    aux1 = aux1+1;
    descres = descres-1;
end
hci = vetorNi(end-1);
%%
cont = 1; % posies do rainflow filtrado
for ii=1:length(RF) % inicio da varredura de todas as posies do rainflow
    % Pontos extremos do ciclo histerese
    ti = RF(ii,4); % tempo de inicio do ciclo de histerese
    tf = RF(ii,5); % tempo final do ciclo de histerese
    xi_pv = find(timepv==ti); % posiao de ti no vetor peaks and valleys (PV)
    xf_pv = find(timepv==tf); % posiao de tf no vetor peaks and valleys (PV)
    xi_full = find(timefull==ti); % posiao de ti no vetor ORIGINAL
    xf_full = find(timefull==tf); % posiao de tf no vetor ORIGINAL
    Ti = Tjfull(xi_full); % temperatura de inicio (ti) do ciclo de histerese
    Tf = Tjfull(xf_full); % temperatura final (tf) do ciclo de histerese
    if RF(ii,1) == 1      
        if Ti < Tf % ciclo crescente
            tbegin = ti; % tempo do primeiro ponto da contagem de ton
            Tbegin = Ti; % temperatura do primeiro ponto da contagem de ton
            Tend = Tf; % temperatura do ultimo ponto da contagem de ton (maxima temperatura da contagem)
            tend = tf; % tempo do ultimo ponto da contagem de ton
%1*******
            %% encontrando o final do ciclo(em que a temperatura  igual a Tbegin
            % procurando pontos anterior e posterior ao ponto dinal no vetor original
            xx = xf_full; % inciando contagem do ponto de temperatura minima
            Taux = Tf;
            while Taux > Ti
                xx = xx +1;
                Taux = Tjfull(xx);
            end

            Tbefore_full = Tjfull(xx-1); % temperatura do ponto anterior a tend
            tbefore_full = timefull(xx-1); % tempo do ponto anterior a tend
            Tafter_full = Tjfull(xx); % temperatura do ponto apos tend
            tafter_full = timefull(xx); % tempo do ponto apos tend
            
            % Calculando tcycle
            % ponto final do ciclo completo
            tend_cycle = (Ti - Tbefore_full)/(Tafter_full-Tbefore_full)*(tafter_full - tbefore_full) + tbefore_full; 
            tcycle = tend_cycle - ti;
            xbefore_end_cycle = xx-1;

            %% Calculando Tmeam
            contador1  = 1;
            %parte crescente
            xx2 = xi_full;
            while xx2 < xf_full
%                 TH = Tjfull(xx2+1);
%                 TL = Tjfull(xx2);
%                 thigh = timefull(xx2+1);
%                 tlow = timefull(xx2);
                dTmeam (contador1) = (Tjfull(xx2+1)+Tjfull(xx2))/2 * (timefull(xx2+1)-timefull(xx2))/tcycle;
                contador1 = contador1+1;
                xx2 = xx2 +1;           
            end
            %parte decrecente
            xx2 = xf_full;
            while xx2 < xbefore_end_cycle
%                 TH = Tjfull(xx2);
%                 TL = Tjfull(xx2+1);
%                 thigh = timefull(xx2);
%                 tlow = timefull(xx2+1);
                dTmeam (contador1) = (Tjfull(xx2+1)+Tjfull(xx2))/2 * (timefull(xx2+1)-timefull(xx2))/tcycle;
                contador1 = contador1+1;
                xx2 = xx2 +1;           
            end
            %ultimo faixa do ciclo 
            TH = Tjfull(xbefore_end_cycle);
            TL = Ti;
%             thigh = timefull(xbefore_end_cycle);
%             tlow = tend_cycle;
            dTmeam (contador1) = (TH+TL)/2 * (tend_cycle-timefull(xbefore_end_cycle))/tcycle;
            
%2******
        else % Ti >= Tf % ciclo decrescente
            tbegin = tf; % tempo do primenro ponto da contagem de ton
            Tbegin = Tf; % temperatura do primenro ponto da contagem de ton
            Tend = Ti; % temperatura do ultimo ponto da contagem de ton (maxima temperatura da contagem)
            
            % encontrando o final do ciclo(em que a temperatura  igual a Tbegin
            % procurando pontos anterior e posterior a tend no vetor original
            xx = xf_full; % inciando contagem do ponto de temperatura minima
            Taux = 0;
            while Taux < Ti
                xx = xx +1;
                Taux = Tjfull(xx);
            end
            Tbefore_full = Tjfull(xx-1); % temperatura do ponto anterior a tend
            tbefore_full = timefull(xx-1); % tempo do ponto anterior a tend
            Tafter_full = Tjfull(xx); % temperatura do ponto apos tend
            tafter_full = timefull(xx); % tempo do ponto apos tend
            
            % Calculando tend
            Tmax = Ti;
            Tmin = Tbefore_full;
            Tafter = Tafter_full;
            tafter = tafter_full;
            tmin = tbefore_full;
            tend = (Tmax - Tmin)/(Tafter - Tmin)*(tafter - tmin) + tmin; % ponto com mesma temperatura de Tmax
%1******
            tend_cycle = tend;
            tcycle = tend_cycle - ti;          
            xbefore_end_cycle = xx-1;

            %% Calculando Tmeam
            contador1  = 1;
            %parte decrecente
            xx2 = xi_full;
            while xx2 < xf_full
%                 TH = Tjfull(xx2);
%                 TL = Tjfull(xx2+1);
%                 thigh = timefull(xx2);
%                 tlow = timefull(xx2+1);
                dTmeam (contador1) = (Tjfull(xx2+1)+Tjfull(xx2))/2 * (timefull(xx2+1)-timefull(xx2))/tcycle;
                %dTmeam (contador1) = (TH+TL)/2 * (thigh-tlow)/tcycle;
                contador1 = contador1+1;
                xx2 = xx2 +1;
            end
            %parte crescente
            xx2 = xf_full;
            while xx2 < xbefore_end_cycle
%                 TH = Tjfull(xx2+1);
%                 TL = Tjfull(xx2);
%                 thigh = timefull(xx2+1);
%                 tlow = timefull(xx2);
                dTmeam (contador1) = (Tjfull(xx2+1)+Tjfull(xx2))/2 * (timefull(xx2+1)-timefull(xx2))/tcycle;
                %dTmeam(contador1) = (TH+TL)/2 * (thigh-tlow)/tcycle;
                contador1 = contador1+1;
                xx2 = xx2 +1;
            end

            %ultimo faixa do ciclo
            TL = Tjfull(xbefore_end_cycle);
            TH = Ti;
%             tlow = timefull(xbefore_end_cycle);
%             thigh = tend_cycle;
            dTmeam (contador1) = (TH+TL)/2 * (tend_cycle-timefull(xbefore_end_cycle))/tcycle;


%2******            
        end
%%
        %% Calculando ton
        % Encontrando picos e vales (PV) dentro da faixa temperatura crescente
        pos_pv = find(timepv>tbegin & timepv<tend); % vector de posioes de PV na faixa crescente
        pic_rest = length(pos_pv); % numero de PV na faixa temperatura crescente
        
        
        if pic_rest == 0 % caso no haja picos
            dtx = tf - ti; % calculando diretamente
            %Tm_cres = (Tf+Ti)/2; %temp media crescente
        else
            % Vectores de PV da faixa temperatura crescente
            vTemp = Tjpv(pos_pv);
            vTime = timepv(pos_pv);
            % incializando variaveis auxiliares
            t_ant = tbegin; % tempo anterior
            Tj_ant = Tbegin; % temperatura anterior
            contador  = 1;
            %iniciando contagem de ton
            for rr=1:length(pos_pv)
                if rr ==1 % primeiro pico (conta-se dtx diretamente)
                    dtx(contador) = vTime(rr)-t_ant; % faixa de tempo aquecendo
                    contador = contador+1;
                    t_ant = vTime(rr);
                    Tj_ant = vTemp(rr);
                elseif pic_rest > 1 && vTemp(rr)< Tj_ant % pontos de PV intermediairos (entre primeiro e ultimo PV)
                    if vTemp(rr+1) > Tj_ant % verificando de o pico posterior  maior que o pico anterior
                        % procutado o proximo ponto tx de temperatura igual a
                        % maxima anterior (Tj_ant)
                        % procurando pontos anterior e posterior a tx no vetor original
                        xx = find(timefull==t_ant); % inciando contagem do ponto de maior temperatura anterior
                        Taux = 0;
                        while Taux < Tj_ant
                            xx = xx +1;
                            Taux = Tjfull(xx);
                        end
                        Tbefore_x_full = Tjfull(xx-1); % temperatura do ponto anterior a tx
                        tbefore_x_full = timefull(xx-1); % tempo do ponto anterior a tx
                        Tafter_x_full = Tjfull(xx); % temperatura do ponto apos tx
                        tafter_x_full = timefull(xx); % tempo do ponto apos tx
                        % Calculando tx
                        Tmax = Tj_ant;
                        Tmin = Tbefore_x_full;
                        Tafter = Tafter_x_full;
                        tafter = tafter_x_full;
                        tmin = tbefore_x_full;
                        tx = (Tmax - Tmin)/(Tafter - Tmin)*(tafter - tmin) + tmin; % ponto com mesma temperatura de Tj_ant
                        dtx(contador) = vTime(rr+1) - tx; % faixa de tempo aquecendo
                        contador = contador+1;
                        t_ant = vTime(rr);
                        Tj_ant = vTemp(rr);
                    end
                elseif pic_rest == 1 && vTemp(rr)< Tj_ant % ultimo vale antes de tend
                    % procutado o proximo ponto tx de temperatura igual a
                    % maxima anterior (Tj_ant)
                    % procurando pontos anterior e posterior a tx no vetor original
                    xx = find(timefull==t_ant); % inciando contagem do ponto de maior temperatura anterior
                    Taux = 0;
                    while Taux < Tj_ant
                        xx = xx +1;
                        Taux = Tjfull(xx);
                    end
                    Tbefore_x_full = Tjfull(xx-1); % temperatura do ponto anterior a tx
                    tbefore_x_full = timefull(xx-1); % tempo do ponto anterior a tx
                    Tafter_x_full = Tjfull(xx); % temperatura do ponto apos tx
                    tafter_x_full = timefull(xx); % tempo do ponto apos tx
                    % Calculando tx
                    Tmax = Tj_ant;
                    Tmin = Tbefore_x_full;
                    Tafter = Tafter_x_full;
                    tafter = tafter_x_full;
                    tmin = tbefore_x_full;
                    tx = (Tmax - Tmin)/(Tafter - Tmin)*(tafter - tmin) + tmin; % ponto com mesma temperatura de Tj_ant
                    dtx(contador) = tend - tx; % ultima faixa de tempo aquecendo
                    contador = contador+1;
                    t_ant = vTime(rr);
                    Tj_ant = vTemp(rr);
                end
                pic_rest = pic_rest-1;
            end
        end
        rf_filt(cont,1) = sum(dtx); % Heating time
        rf_filt(cont,2)= RF(ii,2); % Amplitude (\DeltaT)
        rf_filt(cont,3)= sum(dTmeam); % Mean value (Tm)
%         rf_filt(cont,3)= RF(ii,3); % Mean value (Tm)
        rf_filt(cont,4)= RF(ii,1); % Number of cycles (0.5 or 1.0).
        cont = cont+1;
        clear dtx
        clear dTmeam
        %% %%%%%%%% 0.5 Cycle %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    elseif ii >= hci && Ti < Tf % meio ciclo crescente
        tbegin = ti; % tempo do primeiro ponto da contagem de ton
        Tbegin = Ti; % temperatura do primeiro ponto da contagem de ton
        Tend = Tf; % temperatura do ultimo ponto da contagem de ton (maxima temperatura da contagem)
        tend = tf; % tempo do ultimo ponto da contagem de ton

        tcycle = tend - ti; %meio ciclo

        % Calculando Tmeam
        contador1  = 1;
        %parte crescente
        xx2 = xi_full;
        while xx2 < xf_full
            TH = Tjfull(xx2+1);
            TL = Tjfull(xx2);
            thigh = timefull(xx2+1);
            tlow = timefull(xx2);
            dTmeam(contador1) = (TH+TL)/2 * (thigh-tlow)/tcycle;
            contador1 = contador1+1;
            xx2 = xx2 +1;
        end

%2******

        % Encontrando picos e vales (PV) dentro da faixa temperatura crescente
        pos_pv = find(timepv>tbegin & timepv<tend); % vector de posioes de PV na faixa crescente
        pic_rest = length(pos_pv); % numero de PV na faixa temperatura crescente
        
        if pic_rest == 0 % caso no haja picos
            dtx = tf - ti; % calculando diretamente
        else
            % Vectores de PV da faixa temperatura crescente
            vTemp = Tjpv(pos_pv);
            vTime = timepv(pos_pv);
            % incializando variaveis auxiliares
            t_ant = tbegin; % tempo anterior
            Tj_ant = Tbegin; % temperatura anterior
            contador  = 1;
            %iniciando contagem de ton
            for rr=1:length(pos_pv)
                if rr ==1 % primeiro pico (conta-se dtx diretamente)
                    dtx(contador) = vTime(rr)-t_ant; % faixa de tempo aquecendo
                    contador = contador+1;
                    t_ant = vTime(rr);
                    Tj_ant = vTemp(rr);
                elseif pic_rest ~= 1 && vTemp(rr)< Tj_ant % pontos de PV intermediairos (entre primeiro e ultimo PV)
                    if vTemp(rr+1) > Tj_ant % verificando de o pico posterior  maior que o pico anterior
                        % procutado o proximo ponto tx de temperatura igual a
                        % maxima anterior (Tj_ant)
                        % procurando pontos anterior e posterior a tx no vetor original
                        xx = find(timefull==t_ant); % inciando contagem do ponto de maior temperatura anterior
                        Taux = 0;
                        while Taux < Tj_ant
                            xx = xx +1;
                            Taux = Tjfull(xx);
                        end
                        Tbefore_x_full = Tjfull(xx-1); % temperatura do ponto anterior a tx
                        tbefore_x_full = timefull(xx-1); % tempo do ponto anterior a tx
                        Tafter_x_full = Tjfull(xx); % temperatura do ponto apos tx
                        tafter_x_full = timefull(xx); % tempo do ponto apos tx
                        % Calculando tx
                        Tmax = Tj_ant;
                        Tmin = Tbefore_x_full;
                        Tafter = Tafter_x_full;
                        tafter = tafter_x_full;
                        tmin = tbefore_x_full;
                        tx = (Tmax - Tmin)/(Tafter - Tmin)*(tafter - tmin) + tmin; % ponto com mesma temperatura de Tj_ant
                        dtx(contador) = vTime(rr+1) - tx; % faixa de tempo aquecendo
                        contador = contador+1;
                        t_ant = vTime(rr);
                        Tj_ant = vTemp(rr);
                    end
                elseif pic_rest == 1 && vTemp(rr)< Tj_ant % ultimo vale antes de tend
                    % procutado o proximo ponto tx de temperatura igual a
                    % maxima anterior (Tj_ant)
                    % procurando pontos anterior e posterior a tx no vetor original
                    xx = find(timefull==t_ant); % inciando contagem do ponto de maior temperatura anterior
                    Taux = 0;
                    while Taux < Tj_ant
                        xx = xx +1;
                        Taux = Tjfull(xx);
                    end
                    Tbefore_x_full = Tjfull(xx-1); % temperatura do ponto anterior a tx
                    tbefore_x_full = timefull(xx-1); % tempo do ponto anterior a tx
                    Tafter_x_full = Tjfull(xx); % temperatura do ponto apos tx
                    tafter_x_full = timefull(xx); % tempo do ponto apos tx
                    % Calculando tx
                    Tmax = Tj_ant;
                    Tmin = Tbefore_x_full;
                    Tafter = Tafter_x_full;
                    tafter = tafter_x_full;
                    tmin = tbefore_x_full;
                    tx = (Tmax - Tmin)/(Tafter - Tmin)*(tafter - tmin) + tmin; % ponto com mesma temperatura de Tj_ant
                    dtx(contador) = tend - tx; % ultima faixa de tempo aquecendo
                    contador = contador+1;
                    t_ant = vTime(rr);
                    Tj_ant = vTemp(rr);
                end
                pic_rest = pic_rest-1;
            end
        end
        rf_filt(cont,1) = sum(dtx); % Heating time
        rf_filt(cont,2)= RF(ii,2); % Amplitude (\DeltaT)
        rf_filt(cont,3)= sum(dTmeam); % Mean value (Tm)
        %rf_filt(cont,3)= RF(ii,3); % Mean value (Tm)
        rf_filt(cont,4)= RF(ii,1); % Number of cycles (0.5 or 1.0).
        cont = cont+1;
        clear dtx
        clear dTmeam
    end
end % final da varredura de todas as posies do rainflow